@@ -1,12 +1,17 @@
+# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
+# All rights reserved.
+#
+# This software is provided without warranty under the terms of the BSD
+# license included in LICENSE.txt and may be redistributed only under
+# the conditions described in the aforementioned license. The license
+# is also available online at http://www.enthought.com/licenses/BSD.txt
+#
+# Thanks for using Enthought open source!
 """ The default implementation of a node in a preferences hierarchy. """
 
-from __future__ import print_function
-
 # Standard library imports.
-import logging, threading
-
-# Third-party library imports.
-import six
+import logging
+import threading
 
 # Enthought library imports.
 from traits.api import Any, Callable, Dict, HasTraits, Instance, List
@@ -24,7 +29,6 @@ logger = logging.getLogger(__name__)
 class Preferences(HasTraits):
     """ The default implementation of a node in a preferences hierarchy. """
 
-
     #### 'IPreferences' interface #############################################
 
     # The absolute path to this node from the root node (the empty string if
@@ -86,8 +90,6 @@ class Preferences(HasTraits):
         if len(self.filename) > 0:
             self.load()
 
-        return
-
     ###########################################################################
     # 'IPreferences' interface.
     ###########################################################################
@@ -106,7 +108,7 @@ class Preferences(HasTraits):
 
         names.reverse()
 
-        return '.'.join(names)
+        return ".".join(names)
 
     #### Methods ##############################################################
 
@@ -116,9 +118,9 @@ class Preferences(HasTraits):
         """ Get the value of the preference at the specified path. """
 
         if len(path) == 0:
-            raise ValueError('empty path')
+            raise ValueError("empty path")
 
-        components = path.split('.')
+        components = path.split(".")
 
         # If there is only one component in the path then the operation takes
         # place in this node.
@@ -129,7 +131,7 @@ class Preferences(HasTraits):
         else:
             node = self._get_child(components[0])
             if node is not None:
-                value = node.get('.'.join(components[1:]), Undefined)
+                value = node.get(".".join(components[1:]), Undefined)
 
             else:
                 value = Undefined
@@ -147,7 +149,7 @@ class Preferences(HasTraits):
             del components[-2]
 
             # ... and try that.
-            value = self.get('.'.join(components), default=Undefined)
+            value = self.get(".".join(components), default=Undefined)
 
         if value is Undefined:
             value = default
@@ -158,9 +160,9 @@ class Preferences(HasTraits):
         """ Remove the preference at the specified path. """
 
         if len(path) == 0:
-            raise ValueError('empty path')
+            raise ValueError("empty path")
 
-        components = path.split('.')
+        components = path.split(".")
 
         # If there is only one component in the path then the operation takes
         # place in this node.
@@ -171,17 +173,15 @@ class Preferences(HasTraits):
         else:
             node = self._get_child(components[0])
             if node is not None:
-                node.remove('.'.join(components[1:]))
-
-        return
+                node.remove(".".join(components[1:]))
 
     def set(self, path, value):
         """ Set the value of the preference at the specified path. """
 
         if len(path) == 0:
-            raise ValueError('empty path')
+            raise ValueError("empty path")
 
-        components = path.split('.')
+        components = path.split(".")
 
         # If there is only one component in the path then the operation takes
         # place in this node.
@@ -192,13 +192,11 @@ class Preferences(HasTraits):
         # and pass the rest of the path to that.
         else:
             node = self._node(components[0])
-            node.set('.'.join(components[1:]), value)
-
-        return
+            node.set(".".join(components[1:]), value)
 
     #### Methods where 'path' refers to a node ####
 
-    def clear(self, path=''):
+    def clear(self, path=""):
         """ Remove all preferences from the node at the specified path. """
 
         # If the path is empty then the operation takes place in this node.
@@ -207,15 +205,13 @@ class Preferences(HasTraits):
 
         # Otherwise, find the next node and pass the rest of the path to that.
         else:
-            components = path.split('.')
+            components = path.split(".")
 
             node = self._get_child(components[0])
             if node is not None:
-                node.clear('.'.join(components[1:]))
+                node.clear(".".join(components[1:]))
 
-        return
-
-    def keys(self, path=''):
+    def keys(self, path=""):
         """ Return the preference keys of the node at the specified path. """
 
         # If the path is empty then the operation takes place in this node.
@@ -224,18 +220,18 @@ class Preferences(HasTraits):
 
         # Otherwise, find the next node and pass the rest of the path to that.
         else:
-            components = path.split('.')
+            components = path.split(".")
 
             node = self._get_child(components[0])
             if node is not None:
-                keys = node.keys('.'.join(components[1:]))
+                keys = node.keys(".".join(components[1:]))
 
             else:
                 keys = []
 
         return keys
 
-    def node(self, path=''):
+    def node(self, path=""):
         """ Return the node at the specified path. """
 
         # If the path is empty then the operation takes place in this node.
@@ -244,14 +240,14 @@ class Preferences(HasTraits):
 
         # Otherwise, find the next node and pass the rest of the path to that.
         else:
-            components = path.split('.')
+            components = path.split(".")
 
             node = self._node(components[0])
-            node = node.node('.'.join(components[1:]))
+            node = node.node(".".join(components[1:]))
 
         return node
 
-    def node_exists(self, path=''):
+    def node_exists(self, path=""):
         """ Return True if the node at the specified path exists. """
 
         # If the path is empty then the operation takes place in this node.
@@ -260,20 +256,19 @@ class Preferences(HasTraits):
 
         # Otherwise, find the next node and pass the rest of the path to that.
         else:
-            components = path.split('.')
+            components = path.split(".")
 
             node = self._get_child(components[0])
             if node is not None:
-                exists = node.node_exists('.'.join(components[1:]))
+                exists = node.node_exists(".".join(components[1:]))
 
             else:
                 exists = False
 
         return exists
 
-    def node_names(self, path=''):
-        """ Return the names of the children of the node at the specified path.
-
+    def node_names(self, path=""):
+        """Return the names of the children of the node at the specified path.
         """
 
         # If the path is empty then the operation takes place in this node.
@@ -282,11 +277,11 @@ class Preferences(HasTraits):
 
         # Otherwise, find the next node and pass the rest of the path to that.
         else:
-            components = path.split('.')
+            components = path.split(".")
 
             node = self._get_child(components[0])
             if node is not None:
-                names = node.node_names('.'.join(components[1:]))
+                names = node.node_names(".".join(components[1:]))
 
             else:
                 names = []
@@ -296,7 +291,7 @@ class Preferences(HasTraits):
     #### Persistence methods ####
 
     def flush(self):
-        """ Force any changes in the node to the backing store.
+        """Force any changes in the node to the backing store.
 
         This includes any changes to the node's descendants.
 
@@ -304,50 +299,46 @@ class Preferences(HasTraits):
 
         self.save()
 
-        return
-
     ###########################################################################
     # 'Preferences' interface.
     ###########################################################################
 
     #### Listener methods ####
 
-    def add_preferences_listener(self, listener, path=''):
+    def add_preferences_listener(self, listener, path=""):
         """ Add a listener for changes to a node's preferences. """
 
         # If the path is empty then the operation takes place in this node.
         if len(path) == 0:
-            names = self._add_preferences_listener(listener)
+            self._add_preferences_listener(listener)
 
         # Otherwise, find the next node and pass the rest of the path to that.
         else:
-            components = path.split('.')
+            components = path.split(".")
 
             node = self._node(components[0])
-            node.add_preferences_listener(listener, '.'.join(components[1:]))
+            node.add_preferences_listener(listener, ".".join(components[1:]))
 
-        return
-
-    def remove_preferences_listener(self, listener, path=''):
+    def remove_preferences_listener(self, listener, path=""):
         """ Remove a listener for changes to a node's preferences. """
 
         # If the path is empty then the operation takes place in this node.
         if len(path) == 0:
-            names = self._remove_preferences_listener(listener)
+            self._remove_preferences_listener(listener)
 
         # Otherwise, find the next node and pass the rest of the path to that.
         else:
-            components = path.split('.')
+            components = path.split(".")
 
             node = self._node(components[0])
-            node.remove_preferences_listener(listener,'.'.join(components[1:]))
-
-        return
+            node.remove_preferences_listener(
+                listener, ".".join(components[1:])
+            )
 
     #### Persistence methods ####
 
     def load(self, file_or_filename=None):
-        """ Load preferences from a file.
+        """Load preferences from a file.
 
         This is a *merge* operation i.e. the contents of the file are added to
         the node.
@@ -359,20 +350,20 @@ class Preferences(HasTraits):
         if file_or_filename is None:
             file_or_filename = self.filename
 
-        logger.debug('loading preferences from <%s>', file_or_filename)
+        logger.debug("loading preferences from <%s>", file_or_filename)
 
         # Do the import here so that we don't make 'ConfigObj' a requirement
         # if preferences aren't ever persisted (or a derived class chooses to
         # use a different persistence mechanism).
         from configobj import ConfigObj
 
-        config_obj = ConfigObj(file_or_filename, encoding='utf-8')
+        config_obj = ConfigObj(file_or_filename, encoding="utf-8")
 
         # 'name' is the section name, 'value' is a dictionary containing the
         # name/value pairs in the section (the actual preferences ;^).
         for name, value in config_obj.items():
             # Create/get the node from the section name.
-            components = name.split('.')
+            components = name.split(".")
 
             node = self
             for component in components:
@@ -381,10 +372,8 @@ class Preferences(HasTraits):
             # Add the contents of the section to the node.
             self._add_dictionary_to_node(node, value)
 
-        return
-
     def save(self, file_or_filename=None):
-        """ Save the node's preferences to a file.
+        """Save the node's preferences to a file.
 
         This implementation uses 'ConfigObj' files.
 
@@ -400,14 +389,12 @@ class Preferences(HasTraits):
             # class chooses to use a different persistence mechanism).
             from configobj import ConfigObj
 
-            logger.debug('saving preferences to <%s>', file_or_filename)
+            logger.debug("saving preferences to <%s>", file_or_filename)
 
-            config_obj = ConfigObj(file_or_filename, encoding='utf-8')
+            config_obj = ConfigObj(file_or_filename, encoding="utf-8")
             self._add_node_to_dictionary(self, config_obj)
             config_obj.write()
 
-        return
-
     ###########################################################################
     # Protected 'Preferences' interface.
     #
@@ -425,8 +412,6 @@ class Preferences(HasTraits):
         node._preferences.update(dictionary)
         self._lk.release()
 
-        return
-
     def _add_node_to_dictionary(self, node, dictionary):
         """ Add a node's preferences to a dictionary. """
 
@@ -441,8 +426,6 @@ class Preferences(HasTraits):
         for name in node._node_names():
             self._add_node_to_dictionary(node._get_child(name), dictionary)
 
-        return
-
     def _add_preferences_listener(self, listener):
         """ Add a listener for changes to thisnode's preferences. """
 
@@ -450,8 +433,6 @@ class Preferences(HasTraits):
         self._preferences_listeners.append(listener)
         self._lk.release()
 
-        return
-
     def _clear(self):
         """ Remove all preferences from this node. """
 
@@ -459,8 +440,6 @@ class Preferences(HasTraits):
         self._preferences.clear()
         self._lk.release()
 
-        return
-
     def _create_child(self, name):
         """ Create a child of this node with the specified name. """
 
@@ -480,7 +459,7 @@ class Preferences(HasTraits):
         return value
 
     def _get_child(self, name):
-        """ Return the child of this node with the specified name.
+        """Return the child of this node with the specified name.
 
         Return None if no such child exists.
 
@@ -502,7 +481,7 @@ class Preferences(HasTraits):
         return keys
 
     def _node(self, name):
-        """ Return the child of this node with the specified name.
+        """Return the child of this node with the specified name.
 
         Create the child node if it does not exist.
 
@@ -531,8 +510,6 @@ class Preferences(HasTraits):
             del self._preferences[name]
         self._lk.release()
 
-        return
-
     def _remove_preferences_listener(self, listener):
         """ Remove a listener for changes to the node's preferences. """
 
@@ -541,15 +518,13 @@ class Preferences(HasTraits):
             self._preferences_listeners.remove(listener)
         self._lk.release()
 
-        return
-
     def _set(self, key, value):
         """ Set the value of a preference in this node. """
-        
+
         # everything must be unicode encoded so that ConfigObj configuration
         # can properly serialize the data. Python str are supposed to be ASCII
         # encoded.
-        value = six.text_type(value)
+        value = str(value)
 
         self._lk.acquire()
         old = self._preferences.get(key)
@@ -566,24 +541,18 @@ class Preferences(HasTraits):
         for listener in listeners:
             listener(self, key, old, value)
 
-        return
-
     ###########################################################################
     # Debugging interface.
     ###########################################################################
 
-    def dump(self, indent=''):
+    def dump(self, indent=""):
         """ Dump the preferences hierarchy to stdout. """
 
-        if indent == '':
+        if indent == "":
             print()
 
-        print(indent, 'Node(%s)' % self.name, self._preferences)
-        indent += '  '
+        print(indent, "Node(%s)" % self.name, self._preferences)
+        indent += "  "
 
         for child in self._children.values():
             child.dump(indent)
-
-        return
-
-#### EOF ######################################################################
