@@ -1,3 +1,14 @@
+# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
+# All rights reserved.
+#
+# This software is provided without warranty under the terms of the BSD
+# license included in LICENSE.txt and may be redistributed only under
+# the conditions described in the aforementioned license. The license
+# is also available online at http://www.enthought.com/licenses/BSD.txt
+#
+# Thanks for using Enthought open source!
+
+
 class ProviderNotRegisteredError(Exception):
     """ Raised when a provider is requested by ID and not found. """
 
@@ -28,5 +39,5 @@ class ListenerNotConnectedError(Exception):
         self.listener = listener
 
     def __str__(self):
-        msg = "Selection listener {l} is not connected to provider '{id}'."
-        return msg.format(l=self.listener, id=self.provider_id)
+        msg = "Selection listener {lr} is not connected to provider '{id}'."
+        return msg.format(lr=self.listener, id=self.provider_id)
